use axum::{
    async_trait,
    extract::{FromRef, FromRequestParts},
    http::{request::Parts, StatusCode},
    RequestPartsExt, TypedHeader,
};
use headers::Cookie;
use tracing::error;

use crate::{model::user::User, state::AppState};

pub struct ExtractUserToken(pub Option<User>);

#[async_trait]
impl<S> FromRequestParts<S> for ExtractUserToken
where
    AppState: FromRef<S>,
    S: Send + Sync,
{
    type Rejection = (StatusCode, &'static str);

    async fn from_request_parts(parts: &mut Parts, state: &S) -> Result<Self, Self::Rejection> {
        let state = match parts.extract_with_state::<AppState, _>(state).await {
            Ok(s) => s,
            Err(why) => {
                error!("Getting state: {}", why);
                return Err((StatusCode::INTERNAL_SERVER_ERROR, "Internal server error"));
            }
        };

        let cookie = match parts.extract::<TypedHeader<Cookie>>().await {
            Ok(cookie) => cookie,
            Err(err) => {
                error!("Getting cookie header: {}", err);
                return Ok(ExtractUserToken(None));
            },
        };

        let user = match cookie.get("imgboard-token") {
            Some(_token) => {
                // TODO: get from token
                None
            },
            None => None,
        };

        Ok(ExtractUserToken(user))
    }
}
